import wntr

from magnets.utils import characteristics
from magnets.utils import call_on_functions
from magnets.preprocessing import branch_ends 
from magnets.preprocessing import parallel_pipes
from magnets.reduction import reinitialize
from magnets.reduction import build_g_matrix
from magnets.reduction import mod_reduction

def reduce_model(inp_file, op_pt = None, nodes_to_keep = None, max_nodal_degree = None):
    
    # initialize lists and junction/pipe characteristics
    wn, op_pt,  pipe_dict,junc_dict,unremovable_nodes,special_nodes,special_links,special_links_nodes, relations, new_link_list, results = characteristics(inp_file, op_pt, nodes_to_keep)
    
    # define constants
    alpha = call_on_functions.calculate_unit_coeff(pipe_dict, junc_dict, wn, results, op_pt, special_nodes)
    new_pipe_len = call_on_functions.new_pipe_length(wn, pipe_dict)
    
    # remove branch end nodes and merge parallel pipes
    wn, junc_dict, pipe_dict, relations, new_link_list = branch_ends(relations, wn, new_link_list, junc_dict, pipe_dict, unremovable_nodes)
    wn, junc_dict, pipe_dict, relations, new_link_list = parallel_pipes(relations, wn, new_link_list, junc_dict, pipe_dict, unremovable_nodes, special_nodes, special_links_nodes, special_links, alpha)
       
    # reduce model
    nodes_to_be_removed, link_list_only_junc, unremovable_nodes = reinitialize(wn,new_link_list, special_nodes, unremovable_nodes)
    junc_dict, pipe_dict = build_g_matrix(wn,junc_dict,pipe_dict, unremovable_nodes, special_nodes, alpha)
    mod_reduction(wn, new_link_list, junc_dict, pipe_dict, unremovable_nodes, relations, nodes_to_be_removed, new_pipe_len, alpha, inp_file, max_nodal_degree, op_pt)
    
    return wn
    



"""Top-level package for model_reduction."""

__author__ = """Meghna Sarah Thomas"""
__email__ = 'meghnathomas@utexas.edu'
__version__ = '0.1.5'


from magnets import utils
from magnets import preprocessing
from magnets import reduction



import wntr
from magnets.utils.call_on_functions import *

def build_g_matrix(wn, junc_dict, pipe_dict, unremovable_nodes, special_nodes, alpha):
    
    num_pipes = len(wn.pipe_name_list)
    
    for a in range(num_pipes):
        pipe = wn.pipe_name_list[a]
        if (pipe_dict[pipe]['Start node name'] not in special_nodes and pipe_dict[pipe]['End node name'] not in special_nodes):
            # if (pipe_dict[pipe]['Start node name'] not in unremovable_nodes or pipe_dict[pipe]['End node name'] not in unremovable_nodes):
            nlg = nonlin_g(pipe_dict[pipe]['Length'],pipe_dict[pipe]['Diameter'],pipe_dict[pipe]['Roughness'],alpha)
            lg = nonlin_g_to_lin_g(nlg,junc_dict[pipe_dict[pipe]['Start node name']]['Head at op pt'],junc_dict[pipe_dict[pipe]['End node name']]['Head at op pt'])
            pipe_dict[pipe]['Linear g'] = -lg
            junc_dict[pipe_dict[pipe]['Start node name']]['Diagonal g'] = junc_dict[pipe_dict[pipe]['Start node name']]['Diagonal g'] - pipe_dict[pipe]['Linear g']
            junc_dict[pipe_dict[pipe]['End node name']]['Diagonal g'] = junc_dict[pipe_dict[pipe]['End node name']]['Diagonal g'] - pipe_dict[pipe]['Linear g']
                
    
    return junc_dict, pipe_dict


#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'cycler',
    'decorator',
    'kiwisolver',
    'matplotlib',
    'networkx',
    'numpy',
    'pandas',
    'Pillow',
    'pyparsing',
    'python-dateutil',
    'pytz',
    'scipy',
    'six',
    'wntr>=0.3.0'
]

setup_requirements = [ ]

test_requirements = [ ]

setup(
    author="Meghna Sarah Thomas",
    author_email='meghnathomas@utexas.edu',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="A Python package to aggregate and reduce water distribution network models",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/markdown',
    include_package_data=True,
    keywords='magnets',
    name='magnets',
    packages=find_packages(include=['magnets', 'magnets.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/meghnathomas/magnets',
    version='0.1.5',
    zip_safe=False,
)



import wntr

def branch_ends(relations, wn, new_link_list, junc_dict, pipe_dict, unremovable_nodes):

    num_connections = [len(node) for node in relations.values()]
    
    while min(num_connections) == 1:
        junc_names = wn.junction_name_list
        link_names = wn.link_name_list
        branch_ends = []
        node_indexes = []
        link_indexes = []
        receiving_nodes = []
        
        for i in range(len(num_connections)):
            if num_connections[i] == 1:
                if junc_names[i] not in unremovable_nodes:
                    branch_ends.append(junc_names[i])
                    node_indexes.append(i)
        
        if len(branch_ends) == 0:
            break
        
        for i in range(len(branch_ends)):
            branch_end_node = wn.get_node(branch_ends[i])
            receiving_node_name = relations.get(branch_ends[i])[0]
            receiving_node = wn.get_node(receiving_node_name)
            receiving_nodes.append(receiving_node_name)
            
            #update demand
            junc_dict[receiving_node_name]['Demand'] = junc_dict[receiving_node_name]['Demand'] + junc_dict[branch_ends[i]]['Demand']
            receiving_node.demand_timeseries_list[0].base_value = receiving_node.demand_timeseries_list[0].base_value + branch_end_node.demand_timeseries_list[0].base_value
            
            #delete links and nodes and update G
            for j in range(len(new_link_list)):
                if (new_link_list[j][0] == branch_ends[i] and new_link_list[j][1] == receiving_node_name) or (new_link_list[j][1] == branch_ends[i] and new_link_list[j][0] == receiving_node_name):
                    link_indexes.append(j)
                    
        link_indexes.sort(reverse=True)       
        for k in range(len(link_indexes)):
            wn.remove_link(link_names[link_indexes[k]], force = True)       
            new_link_list.remove(new_link_list[link_indexes[k]])
            del pipe_dict[link_names[link_indexes[k]]]
            
        for l in range(len(branch_ends)):
            wn.remove_node(branch_ends[l], force=True)
            del relations[branch_ends[l]]
            del junc_dict[branch_ends[l]]
            for value in relations.values():
                if branch_ends[l] in value:
                    value.remove(branch_ends[l])
            for m in range(len(receiving_nodes)):
                if branch_ends[l] in junc_dict[receiving_nodes[m]]['Connected nodes']:
                    junc_dict[receiving_nodes[m]]['Connected nodes'].remove(branch_ends[l])
                    
    
        num_connections = [len(node) for node in relations.values()]
        
    return wn, junc_dict, pipe_dict, relations, new_link_list